'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var mock = require('@midwayjs/mock');
var hooksCore = require('@midwayjs/hooks-core');
var path = require('path');

function SuperJSONPlugin(enableSuperjson) {
  if (!enableSuperjson) {
    return () => {};
  }

  return req => {
    req.on('response', res => {
      if (res.body) {
        res.body = hooksCore.superjson.deserialize(res.body);
      }

      if (res.text) {
        res.text = hooksCore.superjson.parse(res.text);
      }
    });
  };
}

async function createAppImplementation(baseDir, isFunction) {
  const root = hooksCore.getProjectRoot(baseDir || global.testPath);
  const config = hooksCore.getConfig(root);
  const cwd = process.cwd();
  process.chdir(root);
  const app = await mock.createApp(root, {
    baseDir: path.join(root, config.source)
  }, isFunction && '@midwayjs/serverless-app');
  process.chdir(cwd);
  return new HooksApplication(app, config);
}
async function createApp(baseDir) {
  return createAppImplementation(baseDir, false);
}
async function createFunctionApp(baseDir) {
  return createAppImplementation(baseDir, true);
}
class HooksApplication {
  constructor(app, config) {
    this.app = void 0;
    this.config = void 0;
    this.app = app;
    this.config = config;
  } // TODO Allow pass user define context


  async runFunction(fn, ...args) {
    const response = await this.request(fn, ...args);

    if (!response.ok) {
      throw response.body;
    }

    if (response.type === 'application/json') {
      return response.body;
    }

    return response.text;
  }

  request(fn, ...args) {
    const supertest = mock.createHttpRequest(this.app);

    if (fn._param.method === 'GET') {
      return supertest.get(fn._param.url).use(SuperJSONPlugin(this.config.superjson));
    }

    return supertest.post(fn._param.url).use(SuperJSONPlugin(this.config.superjson)).send({
      args
    }).set('Content-Type', 'application/json');
  }

  async close() {
    await mock.close(this.app);
  }

}

exports.HooksApplication = HooksApplication;
exports.createApp = createApp;
exports.createAppImplementation = createAppImplementation;
exports.createFunctionApp = createFunctionApp;
